@@ -115,7 +115,8 @@ pub type UsbBusType = UsbBus<Peripheral>;
 /// Enables the Vdd USB power supply. Note that we also need to enable `PWREN` in APB1,
 /// but we handle this using the RTC setup. Use a raw pointer if doing this without the RTC
 /// already set up.
-pub fn enable_usb_pwr(pwr: &mut PWR, rcc: &mut RCC) {
+pub fn enable_usb_pwr() {
     // Enable VddUSB
+    let pwr = unsafe { &*PWR::ptr() };
     pwr.cr2.modify(|_, w| w.usv().set_bit());
 }
